# ==============================================================================
# >> Imports
# ==============================================================================

from cvars import ConVar
from cvars.public import PublicConVar
from config.manager import ConfigManager
from events import Event
from messages import SayText2
from listeners import OnLevelInit, OnConVarChanged
from listeners.tick import Delay
from players.entity import Player
from plugins.info import PluginInfo
from translations.strings import LangStrings

# ==============================================================================
# >> Plugin info
# ==============================================================================
info = PluginInfo("autorespawn")
info.name = "Auto Respawn"
info.basename = "autorespawn"
info.description = "Automatically respawn players."
info.author = "Kill"
info.version = "1.2"
info.url = "https://steamcommunity.com/id/killtmwc"

PublicConVar(info.basename + "_version", info.version, info.description)

# ==============================================================================
# >> Plugin config
# ==============================================================================
cvars = {}  # a dict that holds our cvars

with ConfigManager(info.basename, "autorespawn_") as config:
    config.header = info.name + " plugin configuration file"

    config.text('')  # '\n' doesn't work(as in it doesn't print a new line)
    config.text("To activate a feature set it to 1, to deactivate it, set it to 0(zero)")
    config.text('')

    cvars['autorespawn_enable'] = config.cvar("enable", 1,
                                          "Enable the plugin?")
    cvars['autorespawn_delay'] = config.cvar("delay", 2,
                                         "Time to wait before the player is spawned")
    cvars['autorespawn_showmsg'] = config.cvar("showmsg", 1,
                                           "Show a message to the player before spawning?")
    cvars['autorespawn_unlimited_buytime'] = config.cvar("unlimited_buytime", 1,
                                             "Unlimited buytime?")

# Cvars
ar_enabled = cvars['autorespawn_enable']
ar_delay = cvars['autorespawn_delay'])
ar_showmsg = cvars['autorespawn_showmsg']
ar_buytime = cvars['autorespawn_unlimited_buytime']

# Constants
CHAT_PREFIX = "\x05[SP]\x01"
LANG_STR = LangStrings(info.basename)
RESPAWN_MSG = SayText2(LANG_STR['phrase'])


# Player death event
# todo: (?) add spawn protection
@Event("player_death")
def event_player_death(game_event):
    if ar_enabled.get_bool():
        player = Player.from_userid(game_event.get_int("userid"))
        if ar_showmsg.get_bool():
            RESPAWN_MSG.send(
                player.index,
                prefix=CHAT_PREFIX,
                respawn_delay=ar_delay.get_int()
            )
        Delay(ar_delay.get_int(), player.spawn)


# On map start set the buy time to "unlimited"
# actually the time is set to a very high number, so it's not really unlimited.
# todo: (?) add a check at each round start or player death?
@OnLevelInit
def on_level_init(map):
    if ar_enabled.get_bool() and ar_buytime.get_bool():
        cvar_buytime = ConVar("mp_buytime")
        cvar_buytime.set_float(100000.0)  # If 0(zero) it will use the default value(0.25)
